#!/bin/sh
SATA_RCSID='$Revision: 12195 $ $Date:: 2011-03-06 16:14:38 +0000 #$'

# geninitrd mod: sata

# setup geninitrd module
# @access	public
setup_mod_sata() {
	lspci=$(find_tool /sbin/lspci)

	pcimap="/lib/modules/$kernel/modules.pcimap"

	if [ ! -x "$lspci" ]; then
		warn "Failed to execute lspci. Is pciutils package installed?"
	fi
}

# private until only mod-sata uses the function
find_modules_by_class() {
	local req_class=$1

	# no pcimap, nothing to lookup from
	if [ ! -f "$pcimap" ]; then
		return
	fi

	if [ -z "$lspci" ]; then
		return
	fi

	set -- $($lspci -n | awk -vclass=$req_class '$2 == class":" {split($3, p, ":"); printf("0x0000%s 0x0000%s\n", p[1], p[2])}')

	local PCI_ANY_ID=0x0000ffff pci_module vendor device subvendor subdevice class class_mask driver_data
	while read pci_module vendor device subvendor subdevice class class_mask driver_data; do
		# match class
		[ "$(($req_class & $class_mask))" = "$class" ] || continue
		# match vendor
		[ "$1" = "$vendor" ] || continue
		# match device, allow PCI_ANY_ID
		[ "$2" = "$device" -o $device = $PCI_ANY_ID ] || continue

		echo "$pci_module"
	done < $pcimap
}

find_modules_sata() {
	debug "Finding SATA modules (class=0x0106)"

	# Classes (we want only 0106)
	# http://pci-ids.ucw.cz/read/PD/01
	# 0100 - SCSI
	# 0101 - IDE/PATA
	# 0106 - SATA/AHCI
	# 0107 - SAS
	local m modules=$(find_modules_by_class 0106)

	# sort modules that ones who depend on libata are first
	# this is best we could think of now :)
	local left right
	for m in $modules; do
		if NEW_MODINFO=1 modinfo -k $kernel -F depends $m | grep -q libata; then
			left="$left $m"
		else
			right="$right $m"
		fi
	done

	for m in $left $right; do
		find_module $m
	done
}
